@@ -14,6 +14,7 @@ import glob
 import inspect
 import os
 import platform
+import subprocess
 import sys
 import sysconfig
 import textwrap
@@ -199,6 +200,7 @@ def do_combine_html():
     show_contexts = bool(os.environ.get('COVERAGE_DYNCTX') or os.environ.get('COVERAGE_CONTEXT'))
     cov.html_report(show_contexts=show_contexts)
     cov.xml_report()
+    cov.json_report(pretty_print=True)
 
 
 def do_test_with_tracer(tracer, *runner_args):
@@ -351,6 +353,12 @@ def print_banner(label):
     sys.stdout.flush()
 
 
+def do_quietly(command):
+    """Run a command in a shell, and suppress all output."""
+    proc = subprocess.run(command, shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
+    return proc.returncode
+
+
 def do_help():
     """List the available commands"""
     items = list(globals().items())
