@@ -10,13 +10,12 @@ Some of these are transitional while working toward pure-pytest style.
 import importlib
 import os
 import os.path
-import shutil
 import sys
 
 import pytest
 
 from coverage.misc import SysModuleSaver
-from tests.helpers import change_dir, make_file, remove_files
+from tests.helpers import change_dir, make_file, remove_files, remove_tree
 
 
 class PytestBase:
@@ -82,17 +81,8 @@ class TempDirMixin:
         return make_file(filename, text, bytes, newline)
 
 
-class SysPathModulesMixin:
-    """Auto-restore sys.path and the imported modules at the end of each test."""
-
-    @pytest.fixture(autouse=True)
-    def _save_sys_path(self):
-        """Restore sys.path at the end of each test."""
-        old_syspath = sys.path[:]
-        try:
-            yield
-        finally:
-            sys.path = old_syspath
+class RestoreModulesMixin:
+    """Auto-restore the imported modules at the end of each test."""
 
     @pytest.fixture(autouse=True)
     def _module_saving(self):
@@ -117,9 +107,7 @@ class SysPathModulesMixin:
         # resolution is only one second, a changed file might not be
         # picked up.
         remove_files("*.pyc", "*$py.class")
-        if os.path.exists("__pycache__"):
-            shutil.rmtree("__pycache__")
-
+        remove_tree("__pycache__")
         importlib.invalidate_caches()
 
 
