@@ -462,8 +462,9 @@ class LoopArcTest(CoverageTest):
                 while True:
                     return 1
             """)
-        out = self.run_command("coverage run --branch --source=. main.py")
-        assert out == 'done\n'
+        cov = coverage.Coverage(source=["."], branch=True)
+        self.start_import_stop(cov, "main")
+        assert self.stdout() == 'done\n'
         if env.PYBEHAVIOR.keep_constant_test:
             num_stmts = 3
         elif env.PYBEHAVIOR.nix_while_true:
@@ -471,7 +472,7 @@ class LoopArcTest(CoverageTest):
         else:
             num_stmts = 3
         expected = "zero.py {n} {n} 0 0 0% 1-3".format(n=num_stmts)
-        report = self.report_from_command("coverage report -m")
+        report = self.get_report(cov, show_missing=True)
         squeezed = self.squeezed_lines(report)
         assert expected in squeezed[3]
 
@@ -1529,6 +1530,8 @@ class OptimizedIfTest(CoverageTest):
         arcz_missing = ""
         if env.PYBEHAVIOR.pep626:
             arcz = ".1 12 23 34 42 37 72 28 8."
+        elif env.PYBEHAVIOR.optimize_if_not_debug3:
+            arcz = ".1 12 23 32 37 72 28 8."
         elif env.PYBEHAVIOR.optimize_if_not_debug2:
             arcz = ".1 12 23 35 52 37 72 28 8."
         elif env.PYBEHAVIOR.optimize_if_not_debug:
@@ -1612,28 +1615,25 @@ class MiscArcTest(CoverageTest):
             arcz=".1 15 5A A."
         )
 
-    def test_pathologically_long_code_object(self):
+    @pytest.mark.parametrize("n", [10, 50, 100, 500, 1000, 2000, 10000])
+    def test_pathologically_long_code_object(self, n):
         # https://github.com/nedbat/coveragepy/issues/359
-        # The structure of this file is such that an EXTENDED_ARG bytecode is
-        # needed to encode the jump at the end.  We weren't interpreting those
-        # opcodes.
-        # Note that we no longer interpret bytecode at all, but it couldn't
-        # hurt to keep the test...
-        sizes = [10, 50, 100, 500, 1000, 2000]
-        for n in sizes:
-            code = """\
-                data = [
-                """ + "".join("""\
-                    [
-                        {i}, {i}, {i}, {i}, {i}, {i}, {i}, {i}, {i}, {i}],
-                """.format(i=i) for i in range(n)
-                ) + """\
-                ]
+        # Long code objects sometimes cause problems. Originally, it was
+        # due to EXTENDED_ARG bytes codes.  Then it showed a mistake in
+        # line-number packing.
+        code = """\
+            data = [
+            """ + "".join("""\
+                [
+                    {i}, {i}, {i}, {i}, {i}, {i}, {i}, {i}, {i}, {i}],
+            """.format(i=i) for i in range(n)
+            ) + """\
+            ]
 
-                print(len(data))
-                """
-            self.check_coverage(code, arcs=[(-1, 1), (1, 2*n+4), (2*n+4, -1)])
-        assert self.stdout().split() == [str(n) for n in sizes]
+            print(len(data))
+            """
+        self.check_coverage(code, arcs=[(-1, 1), (1, 2*n+4), (2*n+4, -1)])
+        assert self.stdout() == f"{n}\n"
 
     def test_partial_generators(self):
         # https://github.com/nedbat/coveragepy/issues/475
