@@ -3,7 +3,6 @@
 
 """Test cmdline.py for coverage.py."""
 
-import os
 import pprint
 import sys
 import textwrap
@@ -20,6 +19,7 @@ from coverage.exceptions import ExceptionDuringRun
 from coverage.version import __url__
 
 from tests.coveragetest import CoverageTest, OK, ERR, command_line
+from tests.helpers import os_sep
 
 
 class BaseCmdLineTest(CoverageTest):
@@ -234,6 +234,17 @@ class CmdLineTest(BaseCmdLineTest):
             cov.combine(None, strict=True, keep=False)
             cov.save()
             """)
+        # coverage combine quietly
+        self.cmd_executes("combine -q", """\
+            cov = Coverage(messages=False)
+            cov.combine(None, strict=True, keep=False)
+            cov.save()
+            """)
+        self.cmd_executes("combine --quiet", """\
+            cov = Coverage(messages=False)
+            cov.combine(None, strict=True, keep=False)
+            cov.save()
+            """)
 
     def test_combine_doesnt_confuse_options_with_args(self):
         # https://github.com/nedbat/coveragepy/issues/385
@@ -335,6 +346,79 @@ class CmdLineTest(BaseCmdLineTest):
             cov.load()
             cov.html_report(title='Hello_there')
             """)
+        self.cmd_executes("html -q", """\
+            cov = Coverage(messages=False)
+            cov.load()
+            cov.html_report()
+            """)
+        self.cmd_executes("html --quiet", """\
+            cov = Coverage(messages=False)
+            cov.load()
+            cov.html_report()
+            """)
+
+    def test_json(self):
+        # coverage json [-i] [--omit DIR,...] [FILE1 FILE2 ...]
+        self.cmd_executes("json", """\
+            cov = Coverage()
+            cov.load()
+            cov.json_report()
+            """)
+        self.cmd_executes("json --pretty-print", """\
+            cov = Coverage()
+            cov.load()
+            cov.json_report(pretty_print=True)
+            """)
+        self.cmd_executes("json --pretty-print --show-contexts", """\
+            cov = Coverage()
+            cov.load()
+            cov.json_report(pretty_print=True, show_contexts=True)
+            """)
+        self.cmd_executes("json -i", """\
+            cov = Coverage()
+            cov.load()
+            cov.json_report(ignore_errors=True)
+            """)
+        self.cmd_executes("json -o myjson.foo", """\
+            cov = Coverage()
+            cov.load()
+            cov.json_report(outfile="myjson.foo")
+            """)
+        self.cmd_executes("json -o -", """\
+            cov = Coverage()
+            cov.load()
+            cov.json_report(outfile="-")
+            """)
+        self.cmd_executes("json --omit fooey", """\
+            cov = Coverage(omit=["fooey"])
+            cov.load()
+            cov.json_report(omit=["fooey"])
+            """)
+        self.cmd_executes("json --omit fooey,booey", """\
+            cov = Coverage(omit=["fooey", "booey"])
+            cov.load()
+            cov.json_report(omit=["fooey", "booey"])
+            """)
+        self.cmd_executes("json mod1", """\
+            cov = Coverage()
+            cov.load()
+            cov.json_report(morfs=["mod1"])
+            """)
+        self.cmd_executes("json mod1 mod2 mod3", """\
+            cov = Coverage()
+            cov.load()
+            cov.json_report(morfs=["mod1", "mod2", "mod3"])
+            """)
+        self.cmd_executes("json -q", """\
+            cov = Coverage(messages=False)
+            cov.load()
+            cov.json_report()
+            """)
+        self.cmd_executes("json --quiet", """\
+            cov = Coverage(messages=False)
+            cov.load()
+            cov.json_report()
+            """)
 
     def test_report(self):
         # coverage report [-m] [-i] [-o DIR,...] [FILE1 FILE2 ...]
@@ -700,58 +784,15 @@ class CmdLineTest(BaseCmdLineTest):
             cov.load()
             cov.xml_report(morfs=["mod1", "mod2", "mod3"])
             """)
-
-    def test_json(self):
-        # coverage json [-i] [--omit DIR,...] [FILE1 FILE2 ...]
-        self.cmd_executes("json", """\
-            cov = Coverage()
+        self.cmd_executes("xml -q", """\
+            cov = Coverage(messages=False)
             cov.load()
-            cov.json_report()
+            cov.xml_report()
             """)
-        self.cmd_executes("json --pretty-print", """\
-            cov = Coverage()
+        self.cmd_executes("xml --quiet", """\
+            cov = Coverage(messages=False)
             cov.load()
-            cov.json_report(pretty_print=True)
-            """)
-        self.cmd_executes("json --pretty-print --show-contexts", """\
-            cov = Coverage()
-            cov.load()
-            cov.json_report(pretty_print=True, show_contexts=True)
-            """)
-        self.cmd_executes("json -i", """\
-            cov = Coverage()
-            cov.load()
-            cov.json_report(ignore_errors=True)
-            """)
-        self.cmd_executes("json -o myjson.foo", """\
-            cov = Coverage()
-            cov.load()
-            cov.json_report(outfile="myjson.foo")
-            """)
-        self.cmd_executes("json -o -", """\
-            cov = Coverage()
-            cov.load()
-            cov.json_report(outfile="-")
-            """)
-        self.cmd_executes("json --omit fooey", """\
-            cov = Coverage(omit=["fooey"])
-            cov.load()
-            cov.json_report(omit=["fooey"])
-            """)
-        self.cmd_executes("json --omit fooey,booey", """\
-            cov = Coverage(omit=["fooey", "booey"])
-            cov.load()
-            cov.json_report(omit=["fooey", "booey"])
-            """)
-        self.cmd_executes("json mod1", """\
-            cov = Coverage()
-            cov.load()
-            cov.json_report(morfs=["mod1"])
-            """)
-        self.cmd_executes("json mod1 mod2 mod3", """\
-            cov = Coverage()
-            cov.load()
-            cov.json_report(morfs=["mod1", "mod2", "mod3"])
+            cov.xml_report()
             """)
 
     def test_no_arguments_at_all(self):
@@ -818,7 +859,7 @@ class CmdLineStdoutTest(BaseCmdLineTest):
     @pytest.mark.skipif(env.JYTHON, reason="Jython gets mad if you patch sys.argv")
     def test_help_contains_command_name(self):
         # Command name should be present in help output.
-        fake_command_path = "lorem/ipsum/dolor".replace("/", os.sep)
+        fake_command_path = os_sep("lorem/ipsum/dolor")
         expected_command_name = "dolor"
         fake_argv = [fake_command_path, "sit", "amet"]
         with mock.patch.object(sys, 'argv', new=fake_argv):
@@ -834,7 +875,7 @@ class CmdLineStdoutTest(BaseCmdLineTest):
         # has the `__main__.py` file's patch as the command name. Instead, the command name should
         # be derived from the package name.
 
-        fake_command_path = "lorem/ipsum/dolor/__main__.py".replace("/", os.sep)
+        fake_command_path = os_sep("lorem/ipsum/dolor/__main__.py")
         expected_command_name = "dolor"
         fake_argv = [fake_command_path, "sit", "amet"]
         with mock.patch.object(sys, 'argv', new=fake_argv):
