@@ -16,14 +16,14 @@ import pytest
 
 import coverage
 from coverage import env
-from coverage.exceptions import CoverageException, NotPython, NoSource
+from coverage.exceptions import NotPython, NoSource
 from coverage.files import abs_file, flat_rootname
 import coverage.html
 from coverage.report import get_analysis_to_report
 
 from tests.coveragetest import CoverageTest, TESTS_DIR
 from tests.goldtest import gold_path
-from tests.goldtest import compare, contains, doesnt_contain, contains_any
+from tests.goldtest import compare, contains, contains_rx, doesnt_contain, contains_any
 from tests.helpers import assert_coverage_warnings, change_dir
 
 
@@ -144,6 +144,7 @@ class HtmlDeltaTest(HtmlTestHelpers, CoverageTest):
         self.assert_exists("htmlcov/helper2_py.html")
         self.assert_exists("htmlcov/style.css")
         self.assert_exists("htmlcov/coverage_html.js")
+        self.assert_exists("htmlcov/.gitignore")
 
     def test_html_created(self):
         # Test basic HTML generation: files should be created.
@@ -555,62 +556,6 @@ class HtmlTest(HtmlTestHelpers, CoverageTest):
         self.assert_doesnt_exist("htmlcov/submodule___init___py.html")
 
 
-class HtmlStaticFileTest(CoverageTest):
-    """Tests of the static file copying for the HTML report."""
-
-    def setup_test(self):
-        super().setup_test()
-        original_path = list(coverage.html.STATIC_PATH)
-        self.addCleanup(setattr, coverage.html, 'STATIC_PATH', original_path)
-
-    def test_copying_static_files_from_system(self):
-        # Make a new place for static files.
-        self.make_file("static_here/jquery.min.js", "Not Really JQuery!")
-        coverage.html.STATIC_PATH.insert(0, "static_here")
-
-        self.make_file("main.py", "print(17)")
-        cov = coverage.Coverage()
-        self.start_import_stop(cov, "main")
-        cov.html_report()
-
-        with open("htmlcov/jquery.min.js") as f:
-            jquery = f.read()
-        assert jquery == "Not Really JQuery!"
-
-    def test_copying_static_files_from_system_in_dir(self):
-        # Make a new place for static files.
-        INSTALLED = [
-            "jquery/jquery.min.js",
-            "jquery-hotkeys/jquery.hotkeys.js",
-            "jquery-isonscreen/jquery.isonscreen.js",
-            "jquery-tablesorter/jquery.tablesorter.min.js",
-        ]
-        for fpath in INSTALLED:
-            self.make_file(os.path.join("static_here", fpath), "Not real.")
-        coverage.html.STATIC_PATH.insert(0, "static_here")
-
-        self.make_file("main.py", "print(17)")
-        cov = coverage.Coverage()
-        self.start_import_stop(cov, "main")
-        cov.html_report()
-
-        for fpath in INSTALLED:
-            the_file = os.path.basename(fpath)
-            with open(os.path.join("htmlcov", the_file)) as f:
-                contents = f.read()
-            assert contents == "Not real."
-
-    def test_cant_find_static_files(self):
-        # Make the path point to useless places.
-        coverage.html.STATIC_PATH = ["/xyzzy"]
-
-        self.make_file("main.py", "print(17)")
-        cov = coverage.Coverage()
-        self.start_import_stop(cov, "main")
-        msg = "Couldn't find static file '.*'"
-        with pytest.raises(CoverageException, match=msg):
-            cov.html_report()
-
 def filepath_to_regex(path):
     """Create a regex for scrubbing a file path."""
     regex = re.escape(path)
@@ -623,13 +568,12 @@ def filepath_to_regex(path):
 
 def compare_html(expected, actual, extra_scrubs=None):
     """Specialized compare function for our HTML files."""
+    __tracebackhide__ = True    # pytest, please don't show me this function.
     scrubs = [
         (r'/coverage.readthedocs.io/?[-.\w/]*', '/coverage.readthedocs.io/VER'),
         (r'coverage.py v[\d.abc]+', 'coverage.py vVER'),
         (r'created at \d\d\d\d-\d\d-\d\d \d\d:\d\d [-+]\d\d\d\d', 'created at DATE'),
         (r'created at \d\d\d\d-\d\d-\d\d \d\d:\d\d', 'created at DATE'),
-        # Some words are identifiers in one version, keywords in another.
-        (r'<span class="(nam|key)">(print|True|False)</span>', r'<span class="nam">\2</span>'),
         # Occasionally an absolute path is in the HTML report.
         (filepath_to_regex(TESTS_DIR), 'TESTS_DIR'),
         (filepath_to_regex(flat_rootname(str(TESTS_DIR))), '_TESTS_DIR'),
@@ -638,9 +582,7 @@ def compare_html(expected, actual, extra_scrubs=None):
         (filepath_to_regex(flat_rootname(str(os.getcwd()))), '_TEST_TMPDIR'),
         (filepath_to_regex(abs_file(os.getcwd())), 'TEST_TMPDIR'),
         (filepath_to_regex(flat_rootname(str(abs_file(os.getcwd())))), '_TEST_TMPDIR'),
-        # Old format of test directories that could be in the gold files.
-        (r'/private/var/folders/[\w/]{35}/coverage_test/tests_test_html_\w+_\d{8}', 'TEST_TMPDIR'),
-        (r'_private_var_folders_\w{35}_coverage_test_tests_test_html_\w+_\d{8}', '_TEST_TMPDIR'),
+        (r'/private/var/[\w/]+/pytest-of-\w+/pytest-\d+/(popen-gw\d+/)?t\d+', 'TEST_TMPDIR'),
     ]
     if env.WINDOWS:
         # For file paths...
@@ -669,9 +611,9 @@ class HtmlGoldTest(CoverageTest):
         compare_html(gold_path("html/a"), "out/a")
         contains(
             "out/a/a_py.html",
-            ('<span class="key">if</span> <span class="num">1</span> '
+            ('<span class="key">if</span> <span class="num">1</span> ' +
              '<span class="op">&lt;</span> <span class="num">2</span>'),
-            ('    <span class="nam">a</span> '
+            ('    <span class="nam">a</span> ' +
              '<span class="op">=</span> <span class="num">3</span>'),
             '<span class="pc_cov">67%</span>',
         )
@@ -720,24 +662,24 @@ class HtmlGoldTest(CoverageTest):
         compare_html(gold_path("html/b_branch"), "out/b_branch")
         contains(
             "out/b_branch/b_py.html",
-            ('<span class="key">if</span> <span class="nam">x</span> '
+            ('<span class="key">if</span> <span class="nam">x</span> ' +
              '<span class="op">&lt;</span> <span class="num">2</span>'),
-            ('    <span class="nam">a</span> <span class="op">=</span> '
+            ('    <span class="nam">a</span> <span class="op">=</span> ' +
              '<span class="num">3</span>'),
             '<span class="pc_cov">70%</span>',
 
-            ('<span class="annotate short">3&#x202F;&#x219B;&#x202F;6</span>'
-             '<span class="annotate long">line 3 didn\'t jump to line 6, '
+            ('<span class="annotate short">3&#x202F;&#x219B;&#x202F;6</span>' +
+             '<span class="annotate long">line 3 didn\'t jump to line 6, ' +
                             'because the condition on line 3 was never false</span>'),
-            ('<span class="annotate short">12&#x202F;&#x219B;&#x202F;exit</span>'
-             '<span class="annotate long">line 12 didn\'t return from function \'two\', '
+            ('<span class="annotate short">12&#x202F;&#x219B;&#x202F;exit</span>' +
+             '<span class="annotate long">line 12 didn\'t return from function \'two\', ' +
                             'because the condition on line 12 was never false</span>'),
-            ('<span class="annotate short">20&#x202F;&#x219B;&#x202F;21,&nbsp;&nbsp; '
-                            '20&#x202F;&#x219B;&#x202F;23</span>'
-             '<span class="annotate long">2 missed branches: '
-                            '1) line 20 didn\'t jump to line 21, '
-                                'because the condition on line 20 was never true, '
-                            '2) line 20 didn\'t jump to line 23, '
+            ('<span class="annotate short">20&#x202F;&#x219B;&#x202F;21,&nbsp;&nbsp; ' +
+                            '20&#x202F;&#x219B;&#x202F;23</span>' +
+             '<span class="annotate long">2 missed branches: ' +
+                            '1) line 20 didn\'t jump to line 21, ' +
+                                'because the condition on line 20 was never true, ' +
+                            '2) line 20 didn\'t jump to line 23, ' +
                                 'because the condition on line 20 was never false</span>'),
         )
         contains(
@@ -894,15 +836,15 @@ assert len(math) == 18
             """)
 
         with change_dir("src"):
-            sys.path.insert(0, "")          # pytest sometimes has this, sometimes not!?
             sys.path.insert(0, "../othersrc")
             cov = coverage.Coverage(include=["./*", "../othersrc/*"])
             self.start_import_stop(cov, "here")
             cov.html_report(directory="../out/other")
 
         # Different platforms will name the "other" file differently. Rename it
-        for p in glob.glob("out/other/*_other_py.html"):
-            os.rename(p, "out/other/blah_blah_other_py.html")
+        actual_file = list(glob.glob("out/other/*_other_py.html"))
+        assert len(actual_file) == 1
+        os.rename(actual_file[0], "out/other/blah_blah_other_py.html")
 
         compare_html(
             gold_path("html/other"), "out/other",
@@ -911,9 +853,10 @@ assert len(math) == 18
                 ],
             )
         contains(
-            "out/other/index.html",
+            'out/other/index.html',
             '<a href="here_py.html">here.py</a>',
-            'other_py.html">', 'other.py</a>',
+            'other_py.html">',
+            'other.py</a>',
         )
 
     def test_partial(self):
@@ -951,36 +894,36 @@ assert len(math) == 18
         if env.PYBEHAVIOR.pep626:
             cov.html_report(partial, directory="out/partial_626")
             compare_html(gold_path("html/partial_626"), "out/partial_626")
-            contains(
+            contains_rx(
                 "out/partial_626/partial_py.html",
-                '<p id="t4" class="par run show_par">',
-                '<p id="t7" class="run">',
+                r'<p class="par run show_par">.* id="t4"',
+                r'<p class="run">.* id="t7"',
                 # The "if 0" and "if 1" statements are marked as run.
-                '<p id="t10" class="run">',
+                r'<p class="run">.* id="t10"',
                 # The "raise ZeroDivisionError" is excluded by regex in the .ini.
-                '<p id="t17" class="exc show_exc">',
+                r'<p class="exc show_exc">.* id="t17"',
             )
             contains(
                 "out/partial_626/index.html",
                 '<a href="partial_py.html">partial.py</a>',
-                '<span class="pc_cov">87%</span>'
+                '<span class="pc_cov">87%</span>',
             )
         else:
             cov.html_report(partial, directory="out/partial")
             compare_html(gold_path("html/partial"), "out/partial")
-            contains(
+            contains_rx(
                 "out/partial/partial_py.html",
-                '<p id="t4" class="par run show_par">',
-                '<p id="t7" class="run">',
+                r'<p class="par run show_par">.* id="t4"',
+                r'<p class="run">.* id="t7"',
                 # The "if 0" and "if 1" statements are optimized away.
-                '<p id="t10" class="pln">',
+                r'<p class="pln">.* id="t10"',
                 # The "raise ZeroDivisionError" is excluded by regex in the .ini.
-                '<p id="t17" class="exc show_exc">',
+                r'<p class="exc show_exc">.* id="t17"',
             )
             contains(
                 "out/partial/index.html",
                 '<a href="partial_py.html">partial.py</a>',
-                '<span class="pc_cov">91%</span>'
+                '<span class="pc_cov">91%</span>',
             )
 
     def test_styled(self):
@@ -1003,17 +946,17 @@ assert len(math) == 18
         contains(
             "out/styled/a_py.html",
             '<link rel="stylesheet" href="extra.css" type="text/css">',
-            ('<span class="key">if</span> <span class="num">1</span> '
+            ('<span class="key">if</span> <span class="num">1</span> ' +
              '<span class="op">&lt;</span> <span class="num">2</span>'),
-            ('    <span class="nam">a</span> <span class="op">=</span> '
+            ('    <span class="nam">a</span> <span class="op">=</span> ' +
              '<span class="num">3</span>'),
-            '<span class="pc_cov">67%</span>'
+            '<span class="pc_cov">67%</span>',
         )
         contains(
             "out/styled/index.html",
             '<link rel="stylesheet" href="extra.css" type="text/css">',
             '<a href="a_py.html">a.py</a>',
-            '<span class="pc_cov">67%</span>'
+            '<span class="pc_cov">67%</span>',
         )
 
     def test_tabbed(self):
@@ -1042,10 +985,10 @@ assert len(math) == 18
 
         contains(
             "out/tabbed_py.html",
-            '>        <span class="key">if</span> '
-            '<span class="nam">x</span><span class="op">:</span>'
-            '                                   '
-            '<span class="com"># look nice</span>'
+            '>        <span class="key">if</span> ' +
+            '<span class="nam">x</span><span class="op">:</span>' +
+            '                                   ' +
+            '<span class="com"># look nice</span>',
         )
 
         doesnt_contain("out/tabbed_py.html", "\t")
