@@ -7,7 +7,6 @@ import glob
 import os
 import os.path
 import re
-import shutil
 import stat
 import sys
 import sysconfig
@@ -23,7 +22,7 @@ from coverage.data import line_counts
 from coverage.files import abs_file, python_reported_file
 
 from tests.coveragetest import CoverageTest, TESTS_DIR
-from tests.helpers import change_dir, make_file, nice_file, re_lines, run_command
+from tests.helpers import os_sep, re_lines_text
 
 
 class ProcessTest(CoverageTest):
@@ -39,7 +38,7 @@ class ProcessTest(CoverageTest):
         self.run_command("coverage run mycode.py")
         self.assert_exists(".coverage")
 
-    def test_environment(self):
+    def test_tests_dir_is_importable(self):
         # Checks that we can import modules from the tests directory at all!
         self.make_file("mycode.py", """\
             import covmod1
@@ -53,6 +52,21 @@ class ProcessTest(CoverageTest):
         self.assert_exists(".coverage")
         assert out == 'done\n'
 
+    def test_coverage_run_envvar_is_in_coveragerun(self):
+        # Test that we are setting COVERAGE_RUN when we run.
+        self.make_file("envornot.py", """\
+            import os
+            print(os.environ.get("COVERAGE_RUN", "nope"))
+            """)
+        self.del_environ("COVERAGE_RUN")
+        # Regular Python doesn't have the environment variable.
+        out = self.run_command("python envornot.py")
+        assert out == "nope\n"
+        self.del_environ("COVERAGE_RUN")
+        # But `coverage run` does have it.
+        out = self.run_command("coverage run envornot.py")
+        assert out == "true\n"
+
     def make_b_or_c_py(self):
         """Create b_or_c.py, used in a few of these tests."""
         # "b_or_c.py b" will run 6 lines.
@@ -263,7 +277,6 @@ class ProcessTest(CoverageTest):
         self.assert_exists(".coverage")
         self.assert_file_count(".coverage.*", 2)
 
-
     def test_append_data(self):
         self.make_b_or_c_py()
 
@@ -486,7 +499,7 @@ class ProcessTest(CoverageTest):
         out2 = self.run_command("python throw.py")
         if env.PYPY:
             # Pypy has an extra frame in the traceback for some reason
-            out2 = re_lines(out2, "toplevel", match=False)
+            out2 = re_lines_text("toplevel", out2, match=False)
         assert out == out2
 
         # But also make sure that the output is what we expect.
@@ -830,6 +843,10 @@ class ProcessTest(CoverageTest):
 
     def test_module_name(self):
         # https://github.com/nedbat/coveragepy/issues/478
+        # Make sure help doesn't show a silly command name when run as a
+        # module, like it used to:
+        #   $ python -m coverage
+        #   Code coverage for Python.  Use '__main__.py help' for help.
         out = self.run_command("python -m coverage")
         assert "Use 'coverage help' for help" in out
 
@@ -851,8 +868,8 @@ class EnvironmentTest(CoverageTest):
 
         if env.JYTHON:                  # pragma: only jython
             # Argv0 is different for Jython, remove that from the comparison.
-            expected = re_lines(expected, r'\s+"argv0":', match=False)
-            actual = re_lines(actual, r'\s+"argv0":', match=False)
+            expected = re_lines_text(r'\s+"argv0":', expected, match=False)
+            actual = re_lines_text(r'\s+"argv0":', actual, match=False)
 
         assert actual == expected
 
@@ -888,8 +905,8 @@ class EnvironmentTest(CoverageTest):
         # the comparison also...
         if env.PYPY:
             ignored = re.escape(os.getcwd())
-            expected = re_lines(expected, ignored, match=False)
-            actual = re_lines(actual, ignored, match=False)
+            expected = re_lines_text(ignored, expected, match=False)
+            actual = re_lines_text(ignored, actual, match=False)
         self.assert_tryexecfile_output(expected, actual)
 
     def test_coverage_run_dashm_dir_no_init_is_like_python(self):
@@ -1307,7 +1324,7 @@ class UnicodeFilePathsTest(CoverageTest):
 
         # The XML report is always UTF8-encoded.
         out = self.run_command("coverage xml")
-        assert out == ""
+        assert out == "Wrote XML report to coverage.xml\n"
         with open("coverage.xml", "rb") as xmlf:
             xml = xmlf.read()
         assert ' filename="h\xe2t.py"'.encode() in xml
@@ -1340,7 +1357,7 @@ class UnicodeFilePathsTest(CoverageTest):
         assert expected % os.sep in index
 
         # The XML report is always UTF8-encoded.
-        out = self.run_command("coverage xml")
+        out = self.run_command("coverage xml -q")
         assert out == ""
         with open("coverage.xml", "rb") as xmlf:
             xml = xmlf.read()
@@ -1360,7 +1377,7 @@ class UnicodeFilePathsTest(CoverageTest):
         report_expected = (
             "Name            Stmts   Miss  Cover\n" +
             "-----------------------------------\n" +
-            f"\xe2{os.sep}accented.py       1      0   100%\n" +
+            os_sep("\xe2/accented.py       1      0   100%\n") +
             "-----------------------------------\n" +
             "TOTAL               1      0   100%\n"
         )
@@ -1419,11 +1436,11 @@ def find_writable_pth_directory():
     """Find a place to write a .pth file."""
     for pth_dir in possible_pth_dirs():             # pragma: part covered
         try_it = os.path.join(pth_dir, f"touch_{WORKER}.it")
-        with open(try_it, "w") as f:
-            try:
+        try:
+            with open(try_it, "w") as f:
                 f.write("foo")
-            except OSError:                         # pragma: cant happen
-                continue
+        except OSError:                             # pragma: cant happen
+            continue
 
         os.remove(try_it)
         return pth_dir
@@ -1643,263 +1660,3 @@ class ProcessStartupWithSourceTest(ProcessCoverageMixin, CoverageTest):
 
     def test_script_pkg_sub(self):
         self.assert_pth_and_source_work_together('', 'pkg', 'sub')
-
-
-def run_in_venv(cmd):
-    r"""Run `cmd` in the virtualenv at `venv`.
-
-    The first word of the command will be adjusted to run it from the
-    venv/bin or venv\Scripts directory.
-
-    Returns the text output of the command.
-    """
-    words = cmd.split()
-    if env.WINDOWS:
-        words[0] = fr"venv\Scripts\{words[0]}.exe"
-    else:
-        words[0] = fr"venv/bin/{words[0]}"
-    status, output = run_command(" ".join(words))
-    assert status == 0
-    return output
-
-
-@pytest.fixture(scope="session", name="venv_world")
-def venv_world_fixture(tmp_path_factory):
-    """Create a virtualenv with a few test packages for VirtualenvTest to use.
-
-    Returns the directory containing the "venv" virtualenv.
-    """
-
-    venv_world = tmp_path_factory.mktemp("venv_world")
-    with change_dir(venv_world):
-        # Create a virtualenv.
-        run_command("python -m virtualenv venv")
-
-        # A third-party package that installs a few different packages.
-        make_file("third_pkg/third/__init__.py", """\
-            import fourth
-            def third(x):
-                return 3 * x
-            """)
-        # Use plugin2.py as third.plugin
-        with open(os.path.join(os.path.dirname(__file__), "plugin2.py")) as f:
-            make_file("third_pkg/third/plugin.py", f.read())
-        # A render function for plugin2 to use for dynamic file names.
-        make_file("third_pkg/third/render.py", """\
-            def render(filename, linenum):
-                return "HTML: {}@{}".format(filename, linenum)
-            """)
-        # Another package that third can use.
-        make_file("third_pkg/fourth/__init__.py", """\
-            def fourth(x):
-                return 4 * x
-            """)
-        # Some namespace packages.
-        make_file("third_pkg/nspkg/fifth/__init__.py", """\
-            def fifth(x):
-                return 5 * x
-            """)
-        # The setup.py to install everything.
-        make_file("third_pkg/setup.py", """\
-            import setuptools
-            setuptools.setup(
-                name="third",
-                packages=["third", "fourth", "nspkg.fifth"],
-            )
-            """)
-
-        # Some namespace packages.
-        make_file("another_pkg/nspkg/sixth/__init__.py", """\
-            def sixth(x):
-                return 6 * x
-            """)
-        # The setup.py to install everything.
-        make_file("another_pkg/setup.py", """\
-            import setuptools
-            setuptools.setup(
-                name="another",
-                packages=["nspkg.sixth"],
-            )
-            """)
-
-        # Install the third-party packages.
-        run_in_venv("python -m pip install --no-index ./third_pkg")
-        run_in_venv("python -m pip install --no-index -e ./another_pkg")
-        shutil.rmtree("third_pkg")
-
-        # Install coverage.
-        coverage_src = nice_file(TESTS_DIR, "..")
-        run_in_venv(f"python -m pip install --no-index {coverage_src}")
-
-    return venv_world
-
-
-@pytest.fixture(params=[
-    "coverage",
-    "python -m coverage",
-], name="coverage_command")
-def coverage_command_fixture(request):
-    """Parametrized fixture to use multiple forms of "coverage" command."""
-    return request.param
-
-
-class VirtualenvTest(CoverageTest):
-    """Tests of virtualenv considerations."""
-
-    expected_stdout = "33\n110\n198\n1.5\n"
-
-    @pytest.fixture(autouse=True)
-    def in_venv_world_fixture(self, venv_world):
-        """For running tests inside venv_world, and cleaning up made files."""
-        with change_dir(venv_world):
-            self.make_file("myproduct.py", """\
-                import colorsys
-                import third
-                import nspkg.fifth
-                import nspkg.sixth
-                print(third.third(11))
-                print(nspkg.fifth.fifth(22))
-                print(nspkg.sixth.sixth(33))
-                print(sum(colorsys.rgb_to_hls(1, 0, 0)))
-                """)
-
-            self.del_environ("COVERAGE_TESTING")    # To avoid needing contracts installed.
-            self.set_environ("COVERAGE_DEBUG_FILE", "debug_out.txt")
-            self.set_environ("COVERAGE_DEBUG", "trace")
-
-            yield
-
-            for fname in os.listdir("."):
-                if fname not in {"venv", "another_pkg"}:
-                    os.remove(fname)
-
-    def get_trace_output(self):
-        """Get the debug output of coverage.py"""
-        with open("debug_out.txt") as f:
-            return f.read()
-
-    def test_third_party_venv_isnt_measured(self, coverage_command):
-        out = run_in_venv(coverage_command + " run --source=. myproduct.py")
-        # In particular, this warning doesn't appear:
-        # Already imported a file that will be measured: .../coverage/__main__.py
-        assert out == self.expected_stdout
-
-        # Check that our tracing was accurate. Files are mentioned because
-        # --source refers to a file.
-        debug_out = self.get_trace_output()
-        assert re_lines(
-            debug_out,
-            r"^Not tracing .*\bexecfile.py': inside --source, but is third-party"
-            )
-        assert re_lines(debug_out, r"^Tracing .*\bmyproduct.py")
-        assert re_lines(
-            debug_out,
-            r"^Not tracing .*\bcolorsys.py': falls outside the --source spec"
-            )
-
-        out = run_in_venv("python -m coverage report")
-        assert "myproduct.py" in out
-        assert "third" not in out
-        assert "coverage" not in out
-        assert "colorsys" not in out
-
-    def test_us_in_venv_isnt_measured(self, coverage_command):
-        out = run_in_venv(coverage_command + " run --source=third myproduct.py")
-        assert out == self.expected_stdout
-
-        # Check that our tracing was accurate. Modules are mentioned because
-        # --source refers to a module.
-        debug_out = self.get_trace_output()
-        assert re_lines(
-            debug_out,
-            r"^Not tracing .*\bexecfile.py': " +
-            "module 'coverage.execfile' falls outside the --source spec"
-            )
-        assert re_lines(
-            debug_out,
-            r"^Not tracing .*\bmyproduct.py': module 'myproduct' falls outside the --source spec"
-            )
-        assert re_lines(
-            debug_out,
-            r"^Not tracing .*\bcolorsys.py': module 'colorsys' falls outside the --source spec"
-            )
-
-        out = run_in_venv("python -m coverage report")
-        assert "myproduct.py" not in out
-        assert "third" in out
-        assert "coverage" not in out
-        assert "colorsys" not in out
-
-    def test_venv_isnt_measured(self, coverage_command):
-        out = run_in_venv(coverage_command + " run myproduct.py")
-        assert out == self.expected_stdout
-
-        debug_out = self.get_trace_output()
-        assert re_lines(debug_out, r"^Not tracing .*\bexecfile.py': is part of coverage.py")
-        assert re_lines(debug_out, r"^Tracing .*\bmyproduct.py")
-        assert re_lines(debug_out, r"^Not tracing .*\bcolorsys.py': is in the stdlib")
-
-        out = run_in_venv("python -m coverage report")
-        assert "myproduct.py" in out
-        assert "third" not in out
-        assert "coverage" not in out
-        assert "colorsys" not in out
-
-    @pytest.mark.skipif(not env.C_TRACER, reason="Plugins are only supported with the C tracer.")
-    def test_venv_with_dynamic_plugin(self, coverage_command):
-        # https://github.com/nedbat/coveragepy/issues/1150
-        # Django coverage plugin was incorrectly getting warnings:
-        # "Already imported: ... django/template/blah.py"
-        # It happened because coverage imported the plugin, which imported
-        # Django, and then the Django files were reported as traceable.
-        self.make_file(".coveragerc", "[run]\nplugins=third.plugin\n")
-        self.make_file("myrender.py", """\
-            import third.render
-            print(third.render.render("hello.html", 1723))
-            """)
-        out = run_in_venv(coverage_command + " run --source=. myrender.py")
-        # The output should not have this warning:
-        # Already imported a file that will be measured: ...third/render.py (already-imported)
-        assert out == "HTML: hello.html@1723\n"
-
-    def test_installed_namespace_packages(self, coverage_command):
-        # https://github.com/nedbat/coveragepy/issues/1231
-        # When namespace packages were installed, they were considered
-        # third-party packages.  Test that isn't still happening.
-        out = run_in_venv(coverage_command + " run --source=nspkg myproduct.py")
-        # In particular, this warning doesn't appear:
-        # Already imported a file that will be measured: .../coverage/__main__.py
-        assert out == self.expected_stdout
-
-        # Check that our tracing was accurate. Files are mentioned because
-        # --source refers to a file.
-        debug_out = self.get_trace_output()
-        assert re_lines(
-            debug_out,
-            r"^Not tracing .*\bexecfile.py': " +
-            "module 'coverage.execfile' falls outside the --source spec"
-            )
-        assert re_lines(
-            debug_out,
-            r"^Not tracing .*\bmyproduct.py': module 'myproduct' falls outside the --source spec"
-            )
-        assert re_lines(
-            debug_out,
-            r"^Not tracing .*\bcolorsys.py': module 'colorsys' falls outside the --source spec"
-            )
-
-        out = run_in_venv("python -m coverage report")
-
-        # Name                                                       Stmts   Miss  Cover
-        # ------------------------------------------------------------------------------
-        # another_pkg/nspkg/sixth/__init__.py                            2      0   100%
-        # venv/lib/python3.9/site-packages/nspkg/fifth/__init__.py       2      0   100%
-        # ------------------------------------------------------------------------------
-        # TOTAL                                                          4      0   100%
-
-        assert "myproduct.py" not in out
-        assert "third" not in out
-        assert "coverage" not in out
-        assert "colorsys" not in out
-        assert "fifth" in out
-        assert "sixth" in out
