@@ -33,20 +33,13 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
             a = 1
             print('done')
             """)
-        self.omit_site_packages()
-
-    def omit_site_packages(self):
-        """Write a .coveragerc file that will omit site-packages from reports."""
-        self.make_file(".coveragerc", """\
-            [report]
-            omit = */site-packages/*
-            """)
 
     def test_report(self):
         self.make_mycode()
-        out = self.run_command("coverage run mycode.py")
-        assert out == 'done\n'
-        report = self.report_from_command("coverage report")
+        cov = coverage.Coverage()
+        self.start_import_stop(cov, "mycode")
+        assert self.stdout() == 'done\n'
+        report = self.get_report(cov)
 
         # Name                                           Stmts   Miss  Cover
         # ------------------------------------------------------------------
@@ -65,8 +58,9 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
     def test_report_just_one(self):
         # Try reporting just one module
         self.make_mycode()
-        self.run_command("coverage run mycode.py")
-        report = self.report_from_command("coverage report mycode.py")
+        cov = coverage.Coverage()
+        self.start_import_stop(cov, "mycode")
+        report = self.get_report(cov, morfs=["mycode.py"])
 
         # Name        Stmts   Miss  Cover
         # -------------------------------
@@ -84,6 +78,7 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
     def test_report_wildcard(self):
         # Try reporting using wildcards to get the modules.
         self.make_mycode()
+        # Wildcard is handled by shell or cmdline.py, so use real commands
         self.run_command("coverage run mycode.py")
         report = self.report_from_command("coverage report my*.py")
 
@@ -103,9 +98,9 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
     def test_report_omitting(self):
         # Try reporting while omitting some modules
         self.make_mycode()
-        self.run_command("coverage run mycode.py")
-        omit = f'{TESTS_DIR}/*,*/site-packages/*'
-        report = self.report_from_command(f"coverage report --omit '{omit}'")
+        cov = coverage.Coverage()
+        self.start_import_stop(cov, "mycode")
+        report = self.get_report(cov, omit=[f"{TESTS_DIR}/*", "*/site-packages/*"])
 
         # Name        Stmts   Miss  Cover
         # -------------------------------
@@ -123,8 +118,9 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
     def test_report_including(self):
         # Try reporting while including some modules
         self.make_mycode()
-        self.run_command("coverage run mycode.py")
-        report = self.report_from_command("coverage report --include=mycode*")
+        cov = coverage.Coverage()
+        self.start_import_stop(cov, "mycode")
+        report = self.get_report(cov, include=["mycode*"])
 
         # Name        Stmts   Miss  Cover
         # -------------------------------
@@ -182,9 +178,10 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
                 return x
             branch(1)
             """)
-        out = self.run_command("coverage run --source=. --branch mybranch.py")
-        assert out == 'x\n'
-        report = self.report_from_command("coverage report")
+        cov = coverage.Coverage(source=["."], branch=True)
+        self.start_import_stop(cov, "mybranch")
+        assert self.stdout() == 'x\n'
+        report = self.get_report(cov)
 
         # Name          Stmts   Miss Branch BrPart  Cover
         # -----------------------------------------------
@@ -213,9 +210,10 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
                 return x
             missing(0, 1)
             """)
-        out = self.run_command("coverage run --source=. mymissing.py")
-        assert out == 'y\nz\n'
-        report = self.report_from_command("coverage report --show-missing")
+        cov = coverage.Coverage(source=["."])
+        self.start_import_stop(cov, "mymissing")
+        assert self.stdout() == 'y\nz\n'
+        report = self.get_report(cov, show_missing=True)
 
         # Name           Stmts   Miss  Cover   Missing
         # --------------------------------------------
@@ -237,10 +235,10 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
                     print("y")
             branch(1, 1)
             """)
-        self.omit_site_packages()
-        out = self.run_command("coverage run --branch mybranch.py")
-        assert out == 'x\ny\n'
-        report = self.report_from_command("coverage report --show-missing")
+        cov = coverage.Coverage(branch=True)
+        self.start_import_stop(cov, "mybranch")
+        assert self.stdout() == 'x\ny\n'
+        report = self.get_report(cov, show_missing=True)
 
         # Name           Stmts   Miss Branch BrPart  Cover   Missing
         # ----------------------------------------------------------
@@ -269,11 +267,10 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
                 return x
             branch(1, 1, 0)
             """)
-        self.omit_site_packages()
-        out = self.run_command("coverage run --branch main.py")
-        assert out == 'x\ny\n'
-        report = self.report_from_command("coverage report --show-missing")
-        report_lines = report.splitlines()
+        cov = coverage.Coverage(branch=True)
+        self.start_import_stop(cov, "main")
+        assert self.stdout() == 'x\ny\n'
+        report_lines = self.get_report(cov, squeeze=False, show_missing=True).splitlines()
 
         expected = [
             'Name          Stmts   Miss Branch BrPart  Cover   Missing',
@@ -297,7 +294,7 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
             def not_covered():
                 print("n")
             """)
-        self.omit_site_packages()
+        # --fail-under is handled by cmdline.py, use real commands.
         out = self.run_command("coverage run main.py")
         assert out == "z\n"
         report = self.report_from_command("coverage report --skip-covered --fail-under=70")
@@ -338,10 +335,10 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
                 pass
             foo()
             """)
-        self.omit_site_packages()
-        out = self.run_command("coverage run --branch main.py")
-        assert out == "n\nz\n"
-        report = self.report_from_command("coverage report --skip-covered")
+        cov = coverage.Coverage(branch=True)
+        self.start_import_stop(cov, "main")
+        assert self.stdout() == "n\nz\n"
+        report = self.get_report(cov, skip_covered=True)
 
         # Name             Stmts   Miss Branch BrPart  Cover
         # --------------------------------------------------
@@ -378,10 +375,10 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
             def does_not_appear_in_this_film(ni):
                 print("Ni!")
             """)
-        self.omit_site_packages()
-        out = self.run_command("coverage run --branch main.py")
-        assert out == "n\nz\n"
-        report = self.report_from_command("coverage report --skip-covered")
+        cov = coverage.Coverage(branch=True)
+        self.start_import_stop(cov, "main")
+        assert self.stdout() == "n\nz\n"
+        report = self.get_report(cov, skip_covered=True)
 
         # Name             Stmts   Miss Branch BrPart  Cover
         # --------------------------------------------------
@@ -405,9 +402,10 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
                 pass
             foo()
             """)
-        out = self.run_command("coverage run --source=. --branch main.py")
-        assert out == ""
-        report = self.report_from_command("coverage report --skip-covered")
+        cov = coverage.Coverage(source=["."], branch=True)
+        self.start_import_stop(cov, "main")
+        assert self.stdout() == ""
+        report = self.get_report(cov, skip_covered=True)
 
         # Name      Stmts   Miss Branch BrPart  Cover
         # -------------------------------------------
@@ -426,9 +424,10 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
                 pass
             foo()
             """)
-        out = self.run_command("coverage run --source=. --branch long_______________filename.py")
-        assert out == ""
-        report = self.report_from_command("coverage report --skip-covered")
+        cov = coverage.Coverage(source=["."], branch=True)
+        self.start_import_stop(cov, "long_______________filename")
+        assert self.stdout() == ""
+        report = self.get_report(cov, squeeze=False, skip_covered=True)
 
         # Name    Stmts   Miss Branch BrPart  Cover
         # -----------------------------------------
@@ -461,10 +460,10 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
             normal()
             """)
         self.make_file("submodule/__init__.py", "")
-        self.omit_site_packages()
-        out = self.run_command("coverage run main.py")
-        assert out == "z\n"
-        report = self.report_from_command("coverage report --skip-empty")
+        cov = coverage.Coverage()
+        self.start_import_stop(cov, "main")
+        assert self.stdout() == "z\n"
+        report = self.get_report(cov, skip_empty=True)
 
         # Name             Stmts   Miss  Cover
         # ------------------------------------
@@ -479,14 +478,13 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
         assert squeezed[2] == "main.py 4 0 100%"
         assert squeezed[4] == "TOTAL 4 0 100%"
         assert squeezed[6] == "1 empty file skipped."
-        assert self.last_command_status == 0
 
     def test_report_skip_empty_no_data(self):
         self.make_file("__init__.py", "")
-        self.omit_site_packages()
-        out = self.run_command("coverage run __init__.py")
-        assert out == ""
-        report = self.report_from_command("coverage report --skip-empty")
+        cov = coverage.Coverage()
+        self.start_import_stop(cov, "__init__")
+        assert self.stdout() == ""
+        report = self.get_report(cov, skip_empty=True)
 
         # Name             Stmts   Miss  Cover
         # ------------------------------------
@@ -524,9 +522,10 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
                 pass
             foo()
             """)
-        out = self.run_command("coverage run --branch main.py")
-        assert out == "n\nz\n"
-        report = self.report_from_command("coverage report")
+        cov = coverage.Coverage(branch=True)
+        self.start_import_stop(cov, "main")
+        assert self.stdout() == "n\nz\n"
+        report = self.get_report(cov)
 
         # Name             Stmts   Miss Branch BrPart      Cover
         # ------------------------------------------------------
@@ -653,14 +652,6 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
         report = self.report_from_command("coverage report")
         assert self.last_line_squeezed(report) == "TOTAL 5 0 100%"
 
-    def get_report(self, cov):
-        """Get the report from `cov`, and canonicalize it."""
-        repout = io.StringIO()
-        cov.report(file=repout, show_missing=False)
-        report = repout.getvalue().replace('\\', '/')
-        report = re.sub(r" +", " ", report)
-        return report
-
     def test_bug_156_file_not_run_should_be_zero(self):
         # https://github.com/nedbat/coveragepy/issues/156
         self.make_file("mybranch.py", """\
@@ -674,18 +665,14 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
             print("y")
             """)
         cov = coverage.Coverage(branch=True, source=["."])
-        cov.start()
-        import main     # pragma: nested # pylint: disable=unused-import, import-error
-        cov.stop()      # pragma: nested
+        self.start_import_stop(cov, "main")
         report = self.get_report(cov).splitlines()
         assert "mybranch.py 5 5 2 0 0%" in report
 
     def run_TheCode_and_report_it(self):
         """A helper for the next few tests."""
         cov = coverage.Coverage()
-        cov.start()
-        import TheCode  # pragma: nested # pylint: disable=import-error, unused-import
-        cov.stop()      # pragma: nested
+        self.start_import_stop(cov, "TheCode")
         return self.get_report(cov)
 
     def test_bug_203_mixed_case_listed_twice_with_rc(self):
@@ -716,6 +703,7 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
             print("In mod.pyw")
             """)
         cov = coverage.Coverage()
+        # start_import_stop can't import the .pyw file, so use the long form.
         cov.start()
         import start    # pragma: nested # pylint: disable=import-error, unused-import
         cov.stop()      # pragma: nested
@@ -736,9 +724,7 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
 
         # Run the program.
         cov = coverage.Coverage()
-        cov.start()
-        import main     # pragma: nested # pylint: disable=unused-import, import-error
-        cov.stop()      # pragma: nested
+        self.start_import_stop(cov, "main")
 
         report = self.get_report(cov).splitlines()
         assert "mod.py 1 0 100%" in report
@@ -762,9 +748,7 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
 
         # Run the program.
         cov = coverage.Coverage()
-        cov.start()
-        import main     # pragma: nested # pylint: disable=unused-import, import-error
-        cov.stop()      # pragma: nested
+        self.start_import_stop(cov, "main")
 
         # Put back the missing Python file.
         self.make_file("mod.py", "a = 1\n")
@@ -778,12 +762,7 @@ class SummaryTest(UsingModulesMixin, CoverageTest):
         cov.start()
         import usepkgs  # pragma: nested # pylint: disable=import-error, unused-import
         cov.stop()      # pragma: nested
-
-        repout = io.StringIO()
-        cov.report(file=repout, show_missing=False)
-
-        report = repout.getvalue().replace('\\', '/')
-        report = re.sub(r"\s+", " ", report)
+        report = self.get_report(cov)
         assert "tests/modules/pkg1/__init__.py 1 0 0 0 100%" in report
         assert "tests/modules/pkg2/__init__.py 0 0 0 0 100%" in report
 
@@ -847,13 +826,11 @@ class SummaryReporterConfigurationTest(CoverageTest):
         """
         self.make_rigged_file("file1.py", 339, 155)
         self.make_rigged_file("file2.py", 13, 3)
-        self.make_rigged_file("file3.py", 234, 228)
-        self.make_file("doit.py", "import file1, file2, file3")
+        self.make_rigged_file("file10.py", 234, 228)
+        self.make_file("doit.py", "import file1, file2, file10")
 
         cov = Coverage(source=["."], omit=["doit.py"])
-        cov.start()
-        import doit             # pragma: nested # pylint: disable=import-error, unused-import
-        cov.stop()              # pragma: nested
+        self.start_import_stop(cov, "doit")
         for name, value in options:
             cov.set_option(name, value)
         printer = SummaryReporter(cov)
@@ -871,7 +848,7 @@ class SummaryReporterConfigurationTest(CoverageTest):
         # ------------------------------
         # file1.py     339    155    54%
         # file2.py      13      3    77%
-        # file3.py     234    228     3%
+        # file10.py    234    228     3%
         # ------------------------------
         # TOTAL        586    386    34%
 
@@ -906,30 +883,40 @@ class SummaryReporterConfigurationTest(CoverageTest):
         msg = f"The words {words!r} don't appear in order in {text!r}"
         assert indexes == sorted(indexes), msg
 
+    def test_default_sort_report(self):
+        # Sort the text report by the default (Name) column.
+        report = self.get_summary_text()
+        self.assert_ordering(report, "file1.py", "file2.py", "file10.py")
+
+    def test_sort_report_by_name(self):
+        # Sort the text report explicitly by the Name column.
+        report = self.get_summary_text(('report:sort', 'Name'))
+        self.assert_ordering(report, "file1.py", "file2.py", "file10.py")
+
     def test_sort_report_by_stmts(self):
         # Sort the text report by the Stmts column.
         report = self.get_summary_text(('report:sort', 'Stmts'))
-        self.assert_ordering(report, "file2.py", "file3.py", "file1.py")
+        self.assert_ordering(report, "file2.py", "file10.py", "file1.py")
 
     def test_sort_report_by_missing(self):
         # Sort the text report by the Missing column.
         report = self.get_summary_text(('report:sort', 'Miss'))
-        self.assert_ordering(report, "file2.py", "file1.py", "file3.py")
+        self.assert_ordering(report, "file2.py", "file1.py", "file10.py")
 
     def test_sort_report_by_cover(self):
         # Sort the text report by the Cover column.
         report = self.get_summary_text(('report:sort', 'Cover'))
-        self.assert_ordering(report, "file3.py", "file1.py", "file2.py")
+        self.assert_ordering(report, "file10.py", "file1.py", "file2.py")
 
     def test_sort_report_by_cover_plus(self):
         # Sort the text report by the Cover column, including the explicit + sign.
         report = self.get_summary_text(('report:sort', '+Cover'))
-        self.assert_ordering(report, "file3.py", "file1.py", "file2.py")
+        self.assert_ordering(report, "file10.py", "file1.py", "file2.py")
 
     def test_sort_report_by_cover_reversed(self):
         # Sort the text report by the Cover column reversed.
         report = self.get_summary_text(('report:sort', '-Cover'))
-        self.assert_ordering(report, "file2.py", "file1.py", "file3.py")
+        self.assert_ordering(report, "file2.py", "file1.py", "file10.py")
 
     def test_sort_report_by_invalid_option(self):
         # Sort the text report by a nonsense column.
