@@ -1,4 +1,3 @@
-# coding: utf-8
 # Licensed under the Apache License: http://www.apache.org/licenses/LICENSE-2.0
 # For details: https://github.com/nedbat/coveragepy/blob/master/NOTICE.txt
 
@@ -9,14 +8,15 @@ import os.path
 import re
 from xml.etree import ElementTree
 
-from unittest_mixins import change_dir
+import pytest
 
 import coverage
-from coverage.backward import import_local_file
 from coverage.files import abs_file
+from coverage.misc import import_local_file
 
 from tests.coveragetest import CoverageTest
 from tests.goldtest import compare, gold_path
+from tests.helpers import assert_coverage_warnings, change_dir
 
 
 class XmlTestHelpers(CoverageTest):
@@ -53,13 +53,13 @@ class XmlTestHelpers(CoverageTest):
             return os.path.join(curdir, p)
 
         for i in range(width):
-            next_dir = here("d{}".format(i))
+            next_dir = here(f"d{i}")
             self.make_tree(width, depth-1, next_dir)
         if curdir != ".":
             self.make_file(here("__init__.py"), "")
             for i in range(width):
-                filename = here("f{}.py".format(i))
-                self.make_file(filename, "# {}\n".format(filename))
+                filename = here(f"f{i}.py")
+                self.make_file(filename, f"# {filename}\n")
 
     def assert_source(self, xmldom, src):
         """Assert that the XML has a <source> element with `src`."""
@@ -87,11 +87,11 @@ class XmlTestHelpersTest(XmlTestHelpers, CoverageTest):
         self.assert_source(dom, "something")
         self.assert_source(dom, "another")
 
-        with self.assertRaises(AssertionError):
+        with pytest.raises(AssertionError):
             self.assert_source(dom, "hello")
-        with self.assertRaises(AssertionError):
+        with pytest.raises(AssertionError):
             self.assert_source(dom, "foo")
-        with self.assertRaises(AssertionError):
+        with pytest.raises(AssertionError):
             self.assert_source(dom, "thing")
 
 
@@ -213,7 +213,12 @@ class XmlReportTest(XmlTestHelpers, CoverageTest):
         mod_foo = import_local_file("foo", "src/main/foo.py")                   # pragma: nested
         mod_bar = import_local_file("bar", "also/over/there/bar.py")            # pragma: nested
         cov.stop()                                                              # pragma: nested
-        cov.xml_report([mod_foo, mod_bar])
+        with pytest.warns(Warning) as warns:
+            cov.xml_report([mod_foo, mod_bar])
+        assert_coverage_warnings(
+            warns,
+            "Module not/really was never imported. (module-not-imported)",
+            )
         dom = ElementTree.parse("coverage.xml")
 
         self.assert_source(dom, "src/main")
@@ -274,10 +279,7 @@ class XmlPackageStructureTest(XmlTestHelpers, CoverageTest):
 
     def assert_package_and_class_tags(self, cov, result):
         """Check the XML package and class tags from `cov` match `result`."""
-        self.assertEqual(
-            unbackslash(list(self.package_and_class_tags(cov))),
-            unbackslash(result),
-            )
+        assert unbackslash(list(self.package_and_class_tags(cov))) == unbackslash(result)
 
     def test_package_names(self):
         self.make_tree(width=1, depth=3)
